;;; ExtATT.LSP Aplicacin para extraccin de Atributos de Bloques  
;;; a una Tabla Excel. Ejemplo utilizando ActiveX                  

;;; Inicializacin: Cargar funciones ActiveX y obtener el objeto    
;;; Documento Actual                                                
(vl-load-com)
(setq *EsteDibujo* (vla-get-ActiveDocument (vlax-get-acad-object)))

;;;Verifica si el bloque tiene atributos
(defun CompruebaAtributos (valor)
  (if (not (equal (cdr (nth (atoi valor) *listaBloques*)) "ATRIB"))
    (set_tile "error" "El bloque seleccionado no posee atributos")
    (set_tile "error" "")
  ) ;_ fin de if
) ;_ fin de defun

;;;Funcin asignada al botn Aceptar
(defun ExtrATT (valor)
  (if *listaBloques*
    (if (not (equal (cdr (nth (atoi valor) *listaBloques*)) "ATRIB")
        ) ;_ fin de not
      (set_tile "error"
                "El bloque seleccionado no posee atributos"
      ) ;_ fin de set_tile
      (progn (set_tile "error" "Espere, por favor...")
             (Lista->Excel
               (SelBloque (car (nth (atoi valor) *listaBloques*)))
             ) ;_ fin de Lista->Excel
             (setq *Posicion* (done_dialog 1))
)))) ;_ fin de defun

;;;Funcin que devuelve los integrantes de la Coleccin "BLOCKS".         
;;;Devuelve una lista de los nombres de los objetos que la integran       
(defun ListaNombres (/ ListaBlq)
  (vlax-for obj (vlax-get-property *EsteDibujo* "Blocks")
    (if
      (and
        (not (wcmatch (vlax-get-property obj "Name") "`**,*|*"))
        (equal (vlax-get-property obj "IsXRef") :vlax-false)
      ) ;_ fin de and
      (setq
        listaBlq
         (cons
           (cons
             (vlax-get-property obj "Name")
             (if (TieneAtributos obj) "ATRIB" "")
           ) ;_ fin de cons
           ListaBlq
  )))) ;_ fin de vlax-for
  (setq
    ListaBlq
    (vl-sort listaBlq '(lambda (n1 n2) (< (car n1) (car n2))))
  ) ;_ fin de setq
  ListaBlq
) ;_ fin de defun

;;; Funcin que discrimina entre los bloques con Atributos y los que
;;; no los tienen.                                                  
(defun TieneAtributos (objDefBloque / resultado)
  (vlax-for obj objDefBloque
    (if
      (equal (vlax-get-property obj "ObjectName")
             "AcDbAttributeDefinition"
      ) ;_ fin de equal
      (setq resultado t)
  )) ;_ fin de vlax-for
  resultado
) ;_ fin de defun

;;;Crea el conjunto de selecci y lanza su procesamiento
(defun SelBloque (nombre / ss cont ename listaBloques)
  (setq cont 0)
  (if (setq ss (ssget "X" (list (cons 0 "INSERT") (cons 2 nombre))))
    (while (setq ename (ssname ss cont))
      (setq listaBloques
             (cons (LeeAtribX (vlax-ename->vla-object ename))
                   listaBloques
             ) ;_ fin de cons
      ) ;_ fin de setq
      (setq cont (1+ cont))
    ) ;_ fin de while
  ) ;_ fin de if
  listaBloques
) ;_ fin de defun

;;; Funciones para gestin del cuadro de dilogo                       
(defun CargaDialogo (/ oErr)
  (setq oErr *error* *error* apl-err)
  (if (not *Posicion*)(setq *Posicion* (list -1 -1)))
  (if (minusp (setq dcl_id (load_dialog "EXTRATT.DCL")))
    (alert "No se encuentra el archivo de dilogo EXTRATT.DCL")
    (if (not (new_dialog "extratt" dcl_id "" *Posicion*))
      (alert "No se encuentra la \ndefinicin de dilogo EXTRATT")
      (progn
        (if (setq *listaBloques* (ListaNombres))
          (progn
            (start_list "lista_bloques")
            (mapcar 'add_list
              (mapcar
                '(lambda (term) (strcat (car term) "\t" (cdr term)))
                *listaBloques*) ;_ fin de mapcar
            ) ;_ fin de mapcar
            (end_list)
          ) ;_ fin de progn
          (set_tile "error" "No hay bloques en el Dibujo Actual")
        ) ;_ fin de if
        (action_tile "lista_bloques" "(CompruebaAtributos $value)")
        (action_tile "accept" "(ExtrATT (get_tile \"lista_bloques\"))")
        (action_tile "cancel" "(setq *Posicion* (done_dialog 0))")
        (start_dialog)
        (unload_dialog dcl_id)
      ) ;_ fin de progn
    ) ;_ fin de if
  ) ;_ fin de if
  (setq *error* oErr)
  (princ)
) ;_ fin de defun

;;;Aade un Comando tipo AutoCAD que invoca la funcin                 
(vlax-add-cmd "AtrEX" 'CargaDialogo)

(alert "CadXpress 63\nPrograma de Extraccin de ATRIBUTOS\n\nTeclee ATREX para ejecutar\n\n R. Togores, 2000")